#include <stdio.h>
#include <stdlib.h>
/*
   DSYEV - compute all eigenvalues and, optionally, eigenvectors of a real
           symmetric matrix A (LAPACK)
   ZHEEV - compute all eigenvalues and, optionally, eigenvectors of a complex
           Hermitian matrix A
   Tru64: cc -migrate lapack_test.c -ldxml # not_parallel
   IRIX:  cc lapack_test.c -lscs[_i8][_mp] # [integer8][multi-processor]
   linux: cc -DF2C lapack_test.c -llapack -lf2c  # lapack-3.0 + blas-3.0 (SuSE-8.0)
          cc -DF2C lapack_test.c -llapack -lf2c -lblas -lm -static # nonshared
   SuSE-9.1:   http://rpmseek.com/; rpm -i lapack-3.0-916.src.rpm
          cd /usr/src/packages/SOURCES;tar -xjf lapack-3.0.tar.bz2
          tar -xjf manpages.tar.bz2;cd lapack-3.0;patch -p0 <../lapack-3.0.dif
          make; cp liblapack.{a,so.3.0.0} /usr/lib/
          ln -s /usr/lib/liblapack.so.3.0.0 /usr/lib/liblapack.so.3
   SuSE-10.0-X86-64: lapack-3.0-926=/usr/lib/liblapack.so.3.0.0 32bit
   SuSE-10.3-X86-64:
          
 */
#ifndef DBL
#define DBL 1   /* 1=DBL*8 C, 0=CPLX*16 C++ */
#endif
#ifdef __cplusplus
extern "C" /* avoid special (C++)-names in object files, ex: main__Fv */
#endif
#if DBL
void dsyev_(
        const char *jobz,               /* (in) "N"=NoVectors, "V"=Vectors */
        const char *uplo,               /* (in) "U"=UpperTriangleA, "L"=LowerTA */
        const int *n,                   /* (in) order of matrix */
        double *a,                      /* a[n][lda] (in/out) */
        const int *lda,                 /* (inp) leading dimension >=max(1,n) */
        double *w,                      /* w[n] (out) eigenvalues */
        double *work,                   /* work[lwork] (workspace/out) */
        const int *lwork,               /* (in)   >= max(1,3*n-1) better (nb+2)*n */
        int *info                       /* (out) ==0 on success */
        );
#else
void zheev_(
        const char *jobz,               /* (in) "N"=NoVectors, "V"=Vectors */
        const char *uplo,               /* (in) "U"=UpperTriangleA, "L"=LowerTA */
        const int *n,                   /* (in) order of matrix */
        double *a,                      /* a[n][lda][2] (in/out) */
        const int *lda,                 /* (inp) leading dimension >=max(1,n) */
        double *w,                      /* w[n] (out) eigenvalues */
        double *work,                   /* work[lwork][2] (workspace/out) */
        const int *lwork,               /* (in)   >= max(1,2*n-1) better (nb+2) */
        double *rwork,                  /* rwork[lwork] (workspace) */
        int *info                       /* (out) ==0 on success */
        );
#endif

/* system   n1   dsyev zheev result 
   wf       1000   24s   81s -4658.990435..125203.120225, dsyev im cache?
   wf       1500  133s  262s -5656.412444..187595.155427 
   wf       2000  283s  702s -6577.761139..250259.319939 
   wf       2500  627s 1285s
   wf       3000 1038s 2375s
   wf       3500 1764s 3466s
  => time[s] = 4e-8 * n1^3 (wf: GS160-alpha-731MHz dsyev) 
  => time[s] = 8e-8 * n1^3 (wf: GS160-alpha-731MHz zheev)
*/

int main(){
  int n=2, lda, lwork, info=-1, i, j, cf=2-DBL; /* cfac: dbl=1, cplx=2 */
  double *a, *w, *work, *rwork;
  lda=n;          /* matrix dimension */
  lwork=10*n;     /* (empirical) factor 10 for working space */
  /* allocating necessary memory */
  a=    (double *)malloc(  cf*n*n*sizeof(double));
  w=    (double *)malloc(       n*sizeof(double));
  work= (double *)malloc(cf*lwork*sizeof(double));
  /* now generating a pseudo-random matrix for test and diagonalize it */
#if DBL
  for(i=0;i<n;i++)for(j=0;j<=i;j++) a[i*n+j]=a[j*n+i]=(rand()&255)-2.5;
  dsyev_("V","U",&n,a,&lda,w,work,&lwork,&info);
#else
  rwork=(double *)malloc(  lwork*sizeof(double));
  for(i=0;i<n;i++)for(j=0;j<=i;j++){
    a[(i*n+j)*2  ]=  a[(j*n+i)*2  ]=((rand()&7)-4);
    a[(i*n+j)*2+1]=-(a[(j*n+i)*2+1]=((rand()&2)-1)*1);
  }
  zheev_("V","U",&n,a,&lda,w,work,&lwork,rwork,&info);
#endif
  printf(" info= %d\n w=",info);
  for(i=0;i<n;i++)printf(" %f",w[i]);
#ifdef EV    /* lot of output, only if you want it */
  for(i=0;i<n && i<4;i++){
    printf("\n v%d=",i);
    for(j=0;j<n;j++){
#if DBL
      printf(" % f",a[i*n+j);
#else
      printf(" (% f, % f)",a[(i*n+j)*2],a[(i*n+j)*2+1]);
#endif
    }
  } printf("\n");
#endif
  return 0;
}

#ifdef F2C
/* lapack needs f2c and f2c needs MAIN__() */
/* if you know a better solution, please tell me */
#ifdef __cplusplus
extern "C" /* avoid special (C++)-names in object files, ex: MAIN____Fv */
#endif
int MAIN__(){ return main(); }
#endif
