/*
    using files as memory to handle arrays bigger than mem+swap
    see libc.info (Memory-mapped I/O) for more information
    
    i586-linux-voigt:                 int=4 long=4 dd>4.8GB
    alpha-linux-spinon:               int=4 long=8 mmap>4.8GB (VSZ=4G,RSS=1M)
    MIPS-4/IP25-IRIX64-challenge-n32: int=4 long=4 mmap=2GB
    MIPS-4/IP25-IRIX64-challenge-64:  int=4 long=8 mmap>9GB
    MIPS-4/IP27-IRIX64-thor-64:       int=4 long=8
    alpha-OSF1-wildfire:              int=4 long=8 mmap>4.8GB (VSS=5G,RSS=1M)
    alpha-OSF1-marvel:                int=4 long=8 mmap>10e9B  io=11-38MB/s AdvFS
    
    LFS, Large File Support in Linux, http://www.suse.de/~aj/linux_lfs.html
    gcc -D_FILE_OFFSET_BITS=64 -D_LARGEFILE_SOURCE
    type: off_t=8 (default=4) for lseek, ftell, ..., fseeko, ftello
    
 */
#include <stdio.h>
#include <stdlib.h>

#define HAVE_MMAP 1
#define DEBUG 1

#ifdef HAVE_MMAP
#include <unistd.h>   /* sysconf */
#include <sys/mman.h> /* mmap */

#define BSIZE (1024*1024)  /* big buffer 1MB */
char buff[BSIZE];

/*
   map a memory to a file
 */
void *mymap(FILE *f1,long len,long ofs){
  int fd,dummy=0; long i,pos; void *buf;
  fd=fileno(f1);
  fseek(f1,(size_t)0,SEEK_END); /* fseek after file end failes */
  pos=ftell(f1);
#ifdef DEBUG
  printf(" fd=%d ftell=%ld=%.0e len=%ld=%.0e ...",
    fd,(long)pos,(double)pos,(long)len,(double)len);
  fflush(stdout);
#endif
  /* speed up by buffer (failes on IRIX-6) */
  setvbuf(f1,buff,_IOFBF,BSIZE);
  for(i=0;i<ofs+len-pos;i++) fwrite(&dummy,1,1,f1);
  fflush(f1); /* file expanded */
#ifdef DEBUG
  pos=ftell(f1); printf(" ftell=%ld=%.0e\n",(long)pos,(double)pos);
#endif
  /* the file must have already the size of the map! */
  buf=mmap(NULL,(size_t)len,PROT_READ | PROT_WRITE,MAP_SHARED, fd, (off_t)0);
#ifdef DEBUG
  printf(" mmap=%p\n",buf);
#endif
  /* on error fall back to malloc+load (save+free) */
  if(buf==(void *)-1) fprintf(stderr,"ERROR mmap(fd=%d,len=%.0e) failed",fd,(double)len);
  return buf;
}

int myunmap(void *buf, long len){
  int rc;
  rc=msync(buf,(size_t)len,MS_SYNC); if(rc)fprintf(stderr,"ERROR msync=%d\n",rc);
  rc=munmap(buf,(size_t)len);  if(rc)fprintf(stderr,"ERROR munmap=%d\n",rc);
  return rc;
}

#endif


int main(int argc, char *argv[]){
  FILE *f1; char *buf;
  long len=1024,idx=6;
  size_t page_size = (size_t) sysconf(_SC_PAGESIZE);

  if(argc>=2) len=atol(argv[1]);
  if(argc>=3) idx=atol(argv[2]);
  printf(" int=%d long=%d size_t=%d pagesize=%.2e len=%ld=%.0e\n",
    (int)sizeof(int),(int)sizeof(long),
    (int)sizeof(size_t),(double)page_size,(long)len,(double)len);
  printf(" off_t=%d\n",(int)sizeof(off_t)); 
  f1=fopen("mmap.tmp","r+b");         /* open r/w */
  if(!f1) f1=fopen("mmap.tmp","w+b"); /* create r/w */

  /* the file must have already the size of the map! */
  buf=(char *)mymap(f1,(size_t)len, (off_t)0);

  if(buf==(void *)-1) exit(1);
  printf(" buf[%ld]=%d\n",(long)idx,(int)(buf[idx]));
  buf[idx]++;
  printf(" buf[%ld]=%d\n",(long)idx,(int)(buf[idx]));
//  buf[70]=12;
//  buf[2000]=7;
//  buf[90000]=4;
  system("ps -p $PPID;ls -lh mmap.tmp;sleep 10");

  myunmap(buf,(long)len);
  fclose(f1);
  printf("finished\n");
  return 0;
}
